#include "CommandLineParser.h"
#include <cstdlib>
#include <sstream>

void CommandLineParser::addParameter(string token,string helptext,string defaultvalue)
{
  tokenvaluemap[token]=defaultvalue;
  descriptiontext.append("  -");
  descriptiontext.append(token);
  int space=30-token.length();
  if(space > 0) descriptiontext.append(string(space,' '));
  descriptiontext.append(" - ");
  descriptiontext.append(helptext);
  if(!defaultvalue.empty())
  {
    descriptiontext.append(" (default value: ");
    descriptiontext.append(defaultvalue);
    descriptiontext.append(" )");
  }
  descriptiontext.append("\n");
}

void CommandLineParser::parse(int argc, char **argv,bool zeroArgsForHelp) throw(string)
{
	argc--;
	int last_token=0;
	if(zeroArgsForHelp && (argc == 0)) throw(descriptiontext);
	for(int arg_index=0;arg_index <= argc;arg_index++)
	{
		char *argument=argv[arg_index];
		if(argument[0] == '-') // command line switch?
		{
			if((++argument)[0] == 'h') throw(descriptiontext);
			map<string,string>::iterator it=tokenvaluemap.find(string(argument));
			if(it != tokenvaluemap.end())
			{
				last_token=arg_index+1;
				if(arg_index < argc && (argv[arg_index+1][0] != '-' || (argv[arg_index+1][1] >= '0' && argv[arg_index+1][1] <= '9')) )
				{
					it->second=string(argv[arg_index+1]);
				}
				else
				{
					it->second=string(" "); // empty but set
				}
			}
		}
	}
	for(int i=++last_token;i <= argc;i++)
	{
		remaining_tokens.push_back(string(argv[i]));
	}
}

string CommandLineParser::getArgument(string parameter) throw(string)
{
  map<string,string>::iterator it=tokenvaluemap.find(string(parameter));
  if(it != tokenvaluemap.end() && !it->second.empty())
  {
  	return it->second;
  }
  else
  {
  	throw(string("Missing command line switch -")+parameter+string("!"));
  }
}


long int CommandLineParser::getLongIntArgument(string parameter,int base) throw(string)
{
	string argument=getArgument(parameter);
	char *endptr;
	long int result = strtol(argument.c_str(),&endptr,base);
	if(*endptr != 0) throw(string("Error while converting argument ")+argument+string(" for parameter ")+parameter+string(" to an integer value!"));
	return result;
}

double CommandLineParser::getDoubleArgument(string parameter) throw(string)
{
	string argument=getArgument(parameter);
	char *endptr;
	double result = strtod(argument.c_str(),&endptr);
	if(*endptr != 0) throw(string("Error while converting argument ")+argument+string(" for parameter ")+parameter+string(" to an double value!"));
	return result;
}

list<int> CommandLineParser::getIntegerListArgument(string parameter) throw(string)
{
	list<int> valuelist;
	istringstream parser(getArgument(parameter));
	int tmp;
	char separator;
	while(parser >> tmp) { valuelist.push_back(tmp); parser >> separator; }
	return valuelist;
}

list<double> CommandLineParser::getDoubleListArgument(string parameter) throw(string)
{
	list<double> valuelist;
	istringstream parser(getArgument(parameter));
	double tmp;
	char separator;
	while(parser >> tmp) { valuelist.push_back(tmp); parser >> separator; }
	return valuelist;
}

bool CommandLineParser::getBooleanArgument(string parameter) throw(string)
{
	string argument=getArgument(parameter);
	bool value=false;
	if(argument.size() != 0 && argument[0] != '0')
	{
	  value = ((argument == "on") || (argument == "true"));
	}
	return value;
}
