#include <iostream>
#include <new>
#include <string>
#include "CommandLineParser.h"

using namespace std;

int main(int argc, char **argv)
{
	try{
		CommandLineParser parser("CommandLineParser class example program.");
		parser.addParameter("-name","optional string parameter","MyName");
		parser.addParameter("I","required integer parameter");
		parser.addParameter("-OptionalInteger","optional integer parameter","1");
		parser.addParameter("D","required double parameter");
		parser.addParameter("S","command line switch");
		parser.addParameter("iV","integer parameter (e.g. 0,1,2,3)"); 
		parser.parse(argc,argv,true);
		cout << "--name argument: " << parser.getArgument("-name") << endl
			 << "-I (integer) : " << parser.getLongIntArgument("I") << endl
			 << "--OptionalInteger : " 
			 << parser.getLongIntArgument("-OptionalInteger") << endl
			 << "-D (double) : " << parser.getDoubleArgument("D") << endl
			 << "switch S " 
			 << ((parser.isSet("S")) ? "has" : "hasn't") 
			 << " been set." << endl;  
		list<int> intlist = parser.getIntegerListArgument("iV");
		for(list<int>::const_iterator it=intlist.begin(); it != intlist.end(); it++)
		{
			cout << " integer list entry:  " << *it << endl;
		}
		
		cout << "remaining arguments: " << endl;
	    const list<string> &remaining_args = parser.getRemainingArguments();
	    for(list<string>::const_iterator it=remaining_args.begin(); it != remaining_args.end(); it++)
	    {
	      cout << *it << endl;
	    }
	}
	catch(string msg)
	{
		cerr << msg << endl;
		return -1;
	}	
	catch(bad_alloc e)
	{
		cerr << "Memory allocation failed!" << endl
			 << e.what() << endl;
		return -2;
	}
	catch(...)
	{
		cerr << "An unknown exception occured!" << endl;
		return -3;
	}
	return 0;
}
